import os
import subprocess


class downLoad:
    def __init__(self):
        self.Mount_Save = True

    def Move(self, original, final, identifier):
        if original == "2":
            stepThree = subprocess.Popen(["sudo umount /mnt/mydev"],
                                         shell=True,
                                         stdin=subprocess.PIPE,
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE,
                                         encoding='utf-8')
            stepThree.wait(2)
            if stepThree.poll() == 0:
                print("取消挂载成功！")
                if stepThree.stdout.read() != "":
                    print("输出：", stepThree.stdout.read())
            else:
                print("**取消挂载失败！")
                print("**错误：", stepThree.stderr.read())

            return True
        TAG = "/dev/sda1"
        if not os.path.exists("/mnt/mydev"):
            subp = subprocess.Popen(["sudo mkdir /mnt/mydev"],
                                    shell=True,
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    encoding='utf-8')
            subp.wait(2)
            if subp.poll() == 0:  
                print("挂载文件已经创建成功！")
                if subp.stdout.read() != "":
                    print("输出：", subp.stdout.read())
            else:
                print("**挂载文件创建失败！")
                print("**错误：", subp.stderr.read())
        else:
            print("挂载文件已经存在！")
            pass

        subp = subprocess.Popen(["sudo fdisk -l"],
                                shell=True,
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                encoding='utf-8')
        subp.wait(2)
        if subp.poll() == 0:  
            num = -1
            for index, line in enumerate(subp.stdout.readlines()):
                if line == "磁盘标识符：%s\n" % identifier:
                    num = index + 3
                if index == num:
                    TAG = line[0:9]
                    print("成功获取U盘标志！")
        else:
            print("**U盘标志获取失败！")
            print("**错误：", subp.stderr.read())
            pass

        stepOne = subprocess.Popen(["sudo mount %s /mnt/mydev" % TAG],
                                   shell=True,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   encoding='utf-8')
        stepOne.wait(2)
        if stepOne.poll() == 0:  
            print("挂载成功！")
            if stepOne.stdout.read() != "":
                print("输出：", stepOne.stdout.read())
        else:
            print("**挂载失败！")
            print("**错误：", stepOne.stderr.read())
            return False

        if original == "1":
            return True

        file_path = original.split("/")
        file_name = file_path[len(file_path) - 1]

        file_new_path = final.split("/")
        file_new_name = file_new_path[len(file_new_path) - 1]
        if not os.path.exists("/mnt/mydev/%s" % file_new_name):
            subp = subprocess.Popen(["sudo mkdir /mnt/mydev/%s" % file_new_name],
                                    shell=True,
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    encoding='utf-8')
            subp.wait(2)
            if subp.poll() == 0:  
                print("U盘文件夹已经创建成功！")
                if subp.stdout.read() != "":
                    print("输出：", subp.stdout.read())
            else:
                print("**U盘文件夹创建失败！")
                print("**错误：", subp.stderr.read())
        else:
            print("U盘文件夹已经存在！")
            pass
        stepTwo = subprocess.Popen(["sudo cp %s %s" % (original, final)],
                                   shell=True,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   encoding='utf-8')
        stepTwo.wait(30)
        if stepTwo.poll() == 0:
            print("%s文件复制成功!" % file_name)
            if stepTwo.stdout.read() != "":
                print("输出：", stepTwo.stdout.read())
        else:
            print("**文件复制失败！")
            print("**错误：", stepTwo.stderr.read())

        stepThree = subprocess.Popen(["sudo umount /mnt/mydev"],
                                     shell=True,
                                     stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE,
                                     encoding='utf-8')
        stepThree.wait(2)
        if stepThree.poll() == 0:
            print("取消挂载成功！")
            if stepThree.stdout.read() != "":
                print("输出：", stepThree.stdout.read())
        else:
            print("**取消挂载失败！")
            print("**错误：", stepThree.stderr.read())

        return True


if __name__ == '__main__':
    mMove = downLoad()

    ori = "/home/topeet/test/start_rknn.sh"
    fin = "/mnt/mydev"
    id = "0xc009d7d1"

    flag = mMove.Move(ori, fin, id)

    print("操作标志：", flag)



import datetime

import numpy as np
import serial
import serial.tools.list_ports

out = ['0'] * 100


class Data_matrix:

    def __init__(self):
        self.mixture_A = [['', '', '', '', ''],
                          ['榆树    ', '', '猫上皮  ', '', '葎草    '],
                          ['', '花生    ', '', '蟹      ', ''],
                          ['鸡蛋    ', '', '开/榛   ', '', '西红柿  '],
                          ['', '悬铃木  ', '', '大豆    ', ''],
                          ['小麦    ', '', '普通豚草', '', '鳕鱼    '],
                          ['', '点/烟/交', '', '狗上皮  ', ''],
                          ['蟑螂    ', '', '虾      ', '', '牛奶    '],
                          ['', '屋/粉   ', '', '艾蒿    ', '']]

        self.mixture_B = [['', '', '', '', ''],
                          ['黄/果/黑', '', '猫上皮  ', '', '苦艾    '],
                          ['', '花生    ', '', '分/毛/根', ''],
                          ['蛋清    ', '', '棉絮    ', '', '烟草屑  '],
                          ['', '杨/柳   ', '', '大豆    ', ''],
                          ['蜜蜂毒  ', '', '普通豚草', '', '蒲公英  '],
                          ['', '交链孢霉', '', '狗上皮  ', ''],
                          ['柏树    ', '', '蚊子唾液', '', '牛奶    '],
                          ['', '屋尘螨  ', '', '艾蒿    ', '']]

        self.mixture_C = [['', '', '', '', ''],
                          ['点/分/烟', '', '猫上皮  ', '', '葎草    '],
                          ['', '花生    ', '', '蟹      ', ''],
                          ['鸡蛋    ', '', '普通豚草', '', '鳕/龙/扇'],
                          ['', '柳/杨/榆', '', '大豆    ', ''],
                          ['羊肉    ', '', '虾      ', '', '屋尘    '],
                          ['', '牛肉    ', '', '狗上皮  ', ''],
                          ['蟑螂    ', '', 'CCD     ', '', '牛奶    '],
                          ['', '屋/粉   ', '', '艾蒿    ', '']]

        self.mixture_D = [['', '', '', '', ''],
                          ['', '鳕/鲑/鲈', '', '', ''],
                          ['点/烟/分', '', '猫/狗   ', '', '总IgE   '],
                          ['', '柏/榆/悬', '', '', ''],
                          ['鸡蛋    ', '', '普/艾/苦', '', ''],
                          ['', '牛/羊   ', '', '虾/蟹/扇', ''],
                          ['花/开/腰', '', '屋尘    ', '', ''],
                          ['', '屋/粉   ', '', '芒/菠/苹', ''],
                          ['', '', '牛奶    ', '', '']]

        self.result = [
            ' - ', ' + ', '++ ', '+++'
        ]

ComFlag = False
ComNum = ""

class Em5822_out:
    def __init__(self):
        self.Data = Data_matrix()
        self.Com = ""
        self.ComRun_flag = False

    def em5822_init(self,address):
        self.ports_list = list(serial.tools.list_ports.comports())
        for comport in self.ports_list:
            if list(comport)[1][0:16] == address or list(comport)[1][0:10] == address:
                self.Com = "%s" % comport[0]
                print("串口%s可用！" % self.Com)
            else:
                continue
        if self.Com != "":
            try:  
                self.ser = serial.Serial(self.Com, 9600)  
                self.ComRun_flag = True  
                print("串口初始化完成！")
                return True
            except Exception as e:  
                print("串口不可用！")
                print(e)
                return False
        else:  
            return False

    def em5822_serial(self):
        if len(self.ports_list) <= 0:
            print("无串口设备。")
        else:  
            print("可用的串口设备如下：")
            for comport in self.ports_list:
                print(list(comport)[0], list(comport)[1])

    def comFlagModi(self, new_data):
        self.ComRun_flag = new_data
        print(self.ComRun_flag)

    def data_conversion(self, Base, Naturen, Light):
        sensi_arr = []
        Nat_Flag = np.zeros((9, 5), dtype=int)
        Nat_Flag = Nat_Flag.astype(str)
        Refer_value = np.zeros((9, 5), dtype=float)
        Refer_value = Refer_value.astype(str)

        if Base[4] == "检测组合A":
            sensi_arr = self.Data.mixture_A
        elif Base[4] == "检测组合B":
            sensi_arr = self.Data.mixture_B
        elif Base[4] == "检测组合C":
            sensi_arr = self.Data.mixture_C
        elif Base[4] == "检测组合D":
            sensi_arr = self.Data.mixture_D

        for i in range(len(Naturen)):
            for j in range(len(Naturen[0])):
                if Naturen[i][j] == "阴性":
                    Nat_Flag[i][j] = self.Data.result[0]
                    Naturen[i][j] = "阴性"
                    Refer_value[i][j] = "<0.35"
                elif Naturen[i][j] == "弱阳性":
                    Nat_Flag[i][j] = self.Data.result[1]
                    Refer_value[i][j] = "<0.35"
                elif Naturen[i][j] == "中阳性":
                    Nat_Flag[i][j] = self.Data.result[2]
                    Refer_value[i][j] = "<0.35"
                elif Naturen[i][j] == "强阳性":
                    Nat_Flag[i][j] = self.Data.result[3]
                    Refer_value[i][j] = "<0.35"
                else:
                    Nat_Flag[i][j] = 0
                    Refer_value[i][j] = 0

        return sensi_arr, Nat_Flag, Naturen, Refer_value

    def em5822_out(self, Base, Nature, Data_Light):
        if self.ComRun_flag == False:
            return False
        sensi_arr = []  
        Nat_Flag = []  
        Light = []  
        num = 0  
        Data_Light = np.delete(Data_Light, 0, axis=0)
        sensi_arr, Nat_Flag, Nature, Light = self.data_conversion(Base, Nature, Data_Light)
        out[0] = "        过敏原检验报告单\r\n"  
        out[1] = "姓名：%s      性别:%s\r\n" % (Base[0], Base[1])
        out[2] = "样本号：%s\r\n" % Base[2]
        out[3] = "条码号：%s\r\n" % Base[3]
        out[4] = "样本类型：%s\r\n" % Base[4]
        out[5] = "测试时间：%s\r\n" % Base[5]
        out[6] = "--------------------------------"
        out[7] = "过敏原     结果  参考值 结果解释\r\n"
        for i in range(45):
            row = int(i / 5)  
            column = int(i % 5)  
            if sensi_arr[row][column] != "":
                num += 1
                out[7 + num] = "%02d%s  %s  %s   %03s\r\n" % (num, sensi_arr[row][column], Nat_Flag[row][column],
                                                              Light[row][column], Nature[row][column])
        out[7 + num + 1] = "————————————————"
        out[7 + num + 2] = "注：\r\n"
        out[7 + num + 3] = "'-'为阴性，<0.35IU/mL\r\n"
        out[7 + num + 4] = "'+'为弱阳性，0.35IU/mL-3.5IU/mL\r\n"
        out[7 + num + 5] = "'++'为中阳性，3.5IU/mL-17.5IU/mL\r\n"
        out[7 + num + 6] = "'+++'为强阳性，≥17.5IU/mL\r\n"
        out[7 + num + 7] = "--------------------------------"
        out[7 + num + 8] = "打印时间：%s\r\n" % Base[6]
        out[7 + num + 9] = "    此检疫报告只对此标本负责，请结合临床。\r\n"
        out[7 + num + 10] = "\r\n"
        out[7 + num + 11] = "\r\n"

        try:
            for i in range(7 + num + 12):
                Print_flag = 2  
                if Print_flag == 0:
                    if i >= 6 and i <= 27:
                        continue
                elif Print_flag == 1:
                    if (i < 6) or (i > 10 and i < 7 + num + 12 - 2):
                        continue
                print(out[i])  
                self.ser.write(out[i].encode("GBK"))  
            print("成功完成打印工作！")
            return True
        except Exception as e:
            print(e)
            print("未完成打印工作！")
            return False


if __name__ == '__main__':
    now = datetime.datetime.now()
    time_now = now.strftime("%Y-%m-%d %H:%M:%S")
    Data_Base = ["路人甲", "男", "0123456789", "9876543210", "检测组合A", time_now, time_now]
    Data_Nature = [['0', '0', '强阳性', '0', '0'],
                   ['弱阳性', '弱阳性', '弱阳性', '阴性', '弱阳性'],
                   ['弱阳性', '弱阳性', '阴性', '阴性', '弱阳性'],
                   ['强阳性', '弱阳性', '中阳性', '弱阳性', '弱阳性'],
                   ['强阳性', '弱阳性', '中阳性', '弱阳性', '弱阳性'],
                   ['弱阳性', '中阳性', '弱阳性', '弱阳性', '阴性'],
                   ['阴性', '中阳性', '阴性', '阴性', '阴性'],
                   ['阴性', '阴性', '阴性', '阴性', '阴性'],
                   ['阴性', '阴性', '阴性', '阴性', '阴性']]
    Data_Light = [[1095533, -298002, 1365723, -298002, 1372170],
                  [92754, 66682, 68361, 55653, 98561],
                  [94383, 84880, 50138, 57907, 75579],
                  [1375053, 343235, 924575, 383978, 187809],
                  [1375053, 328488, 884595, 391545, 187892],
                  [88829, 1042217, 73020, 72565, 26640],
                  [34906, 1046234, 41228, 32066, 20377],
                  [12650, 53237, 10125, 8014, 5839],
                  [0, 9031, 6793, 2547, 3514]]
    ePrint = Em5822_out()



import datetime
import os
import random
import sys
import cv2 as cv
import numpy as np

sys.path.append(os.path.dirname(__file__))

from SecBase import Img_base
from SecBase import Img_show
from SecCore import Img_core


class stageSec:

    def __init__(self):
        self.Base = Img_base()
        self.Show = Img_show()
        self.Core = Img_core()

        self.Print_Save = True

    def process(self, path_read, path_write, combina, radius):
        print("_______________________________________________")
        print("0    前期参数设置")
        gray_aver = np.zeros((9, 5), dtype=int)  
        nature_aver = np.zeros((9, 5), dtype=int)  
        nature_aver = nature_aver.astype(str)
        img_ori = self.Show.img_read(path_read)
        cv.imwrite(path_write + 'img_0ori.jpeg', img_ori)
        dst_init = self.Base.get_gray_round(img_ori)
        print("当前环境灰度值为：", self.Base.gray_round)
        print("_______________________________________________")
        print("1    获取区域内的定位点")
        num_flag = 0
        while True:
            num_flag += 1  
            dst_init -= 10  
            self.gray_value = dst_init
            img_dst = self.Show.img_dst(img_ori, dst_init)
            img_dst = self.Show.img_erosion(img_dst, 3)
            cv.imwrite(path_write + 'img_1dst.jpeg', img_dst)
            judge, img_gray, circle, circle_x, circle_y = self.Core.img_location_first(img_ori, img_dst, num=20, flag=0)
            if judge == 1:
                cv.imwrite(path_write + 'img_2ROI.jpeg', img_gray)
                print("检测到定位点，阈值为：", dst_init)
                break
            elif judge == 0:
                print("阈值为%03s" % dst_init + "\t" + "未检测到定位点")
            if num_flag >= 10:
                cv.imwrite(path_write + 'img_final.jpeg', img_ori)
                print("**错误：难以准确识别定位点")
                return False, gray_aver, nature_aver
            else:
                continue
        print("_______________________________________________")
        print("2    初次矫正图像角度")
        img_rota, img_rota_dst, middle_index = self.Core.img_correct_first(img_ori, img_dst, circle_x, circle_y)

        print("_______________________________________________")
        print("3    再次矫正图像角度")
        point_x, point_y, dis_error, locat_x, locat_y = self.Core.img_correct_second(img_rota, img_rota_dst, circle_x,
                                                                                     circle_y, flag=1)
        if locat_x == circle_x and locat_y == circle_y:
            return False, gray_aver, nature_aver
        print("_______________________________________________")
        print("4    圈定试剂点")
        gray_aver, nature_aver, img_rota, judge_1 = self.Core.img_get_gray(img_rota, gray_aver, nature_aver,
                                                                           locat_x, locat_y, point_x, point_y,
                                                                           dis_error, radius)

        font = cv.FONT_HERSHEY_SIMPLEX
        img_rota = cv.putText(img_rota, "Gray_Threshold: %s" % (self.gray_value), (50, 120), font, 3, (255, 255, 255),
                              6)
        img_rota = cv.putText(img_rota, "ROI_Down: %s" % (self.Core.gray_down), (50, 250), font, 3, (255, 255, 255), 6)
        img_rota = cv.putText(img_rota, "Dip_Angle: %.3f" % (self.Core.dip_angle), (50, 380), font, 3, (255, 255, 255),
                              6)
        img_rota = cv.putText(img_rota, "Gray_Surrounding: %.3f" % (self.Base.gray_round), (50, 510), font, 3,
                              (255, 255, 255), 6)

        cv.imwrite(path_write + 'img_final.jpeg', img_rota)

        now = datetime.datetime.now()
        time_now = now.strftime("%Y-%m-%d_%H-%M-%S")
        path_split = path_write.split('/')
        path_split[len(path_split) - 2] = "img_history"
        path_history = ""
        for i in range(len(path_split)):
            path_history = os.path.join(path_history, path_split[i])
        path_split[len(path_split) - 2] = "img_input"
        path_input = ""
        for i in range(len(path_split)):
            path_input = os.path.join(path_input, path_split[i])
        cv.imwrite(path_history + "%s_ori.jpeg" % time_now, img_ori)
        cv.imwrite(path_history + "%s_fin.jpeg" % time_now, img_rota)

        print("_______________________________________________")
        print("5    输出最终数据")
        print("参数-二值化阈值： %s" % (self.gray_value))
        print("参数-ROI下移区间： %s" % (self.Core.gray_down))
        print("参数-图像矫正角度： %.3f" % (self.Core.dip_angle))
        print("参数-环境灰度值： %.3f" % (self.Base.gray_round))
        print("发光矩阵：\r", gray_aver)
        print("性质矩阵：\r", nature_aver)

        return True, gray_aver, nature_aver



import math
import os
import operator

import datetime
import cv2 as cv
import numpy as np

import SecBase


class Img_core:

    def __init__(self):
        self.Base = SecBase.Img_base()
        self.script = os.path.dirname(os.path.abspath(__file__))

        self.gray_down = 0
        self.dip_angle = 0

    def img_location_first(self, img_gray, img_dst, num, flag):
        
        num_down = 0  
        num_init = 500  
        len_down = 50  
        circle_x = []  
        circle_y = []  
        circle_r = []  
        for i in range(num):
            num_down = len_down * i + num_init  
            img_ROI = img_dst[(self.Base.roiPos[0][0] + num_down):(self.Base.roiPos[0][1] + num_down),
                      self.Base.roiPos[1][0]:self.Base.roiPos[1][1]]

            circle = cv.HoughCircles(img_ROI, cv.HOUGH_GRADIENT, 0.5, 400, param1=100, param2=8, minRadius=50,
                                     maxRadius=150)
            if circle is None:
                continue
            elif circle.shape[1] < 3:
                continue
            elif circle.shape[1] >= 3:
                for i in range(circle.shape[1]):
                    circle[0][i][0] = circle[0][i][0] + self.Base.roiPos[1][0]
                    circle[0][i][1] = circle[0][i][1] + self.Base.roiPos[0][0] + num_down
                for j in circle[0, :]:
                    circle_x.append(j[0])
                    circle_y.append(j[1])
                    circle_r.append(j[2])

                exit_flag = 0

                circle_y_sort = sorted(circle_y, key=float)  
                diff_x1 = np.diff(circle_y_sort)  
                for i in range(len(diff_x1)):  
                    if diff_x1[i] > 200:
                        exit_flag = 1
                        break
                if exit_flag != 0:
                    print("---------------------")
                    print("区间下移量：", num_down)
                    print("退出标志为：Y轴像素误差大于200")
                    circle_x.clear()
                    circle_y.clear()
                    circle_r.clear()
                    continue

                circle_x_sort = sorted(circle_x, key=float)  
                diff_x2 = np.diff(circle_x_sort)  
                for i in range(len(diff_x2)):  
                    if diff_x2[i] < 630:
                        exit_flag = 1
                        break
                if exit_flag != 0:
                    print("---------------------")
                    print("区间下移量：", num_down)
                    print("退出标志为：X轴像素误差小于630")
                    circle_x.clear()
                    circle_y.clear()
                    circle_r.clear()
                    continue

                if circle.shape[1] == 3:  
                    gray_posi = np.zeros(1 * 3)  
                    img_array = np.transpose(np.array(img_gray))  
                    for j in range(3):
                        gray_posi[j] = self.Base.sum_gray_ave(img_array,
                                                              int(circle_x[j]),
                                                              int(circle_y[j]),
                                                              int(circle_r[j]))
                    min_index, min_number = min(enumerate(gray_posi), key=operator.itemgetter(1))
                    max_index, max_number = max(enumerate(gray_posi), key=operator.itemgetter(1))
                    if (max_number - min_number) >= 20:  
                        exit_flag = 1
                else:
                    gray_posi = np.zeros(1 * circle.shape[1])
                    exit_flag = 1

                if exit_flag != 0:
                    print("---------------------")
                    print("区间下移量：", num_down)
                    print("退出标志为：灰度均值不一致")
                    circle_x.clear()
                    circle_y.clear()
                    circle_r.clear()
                    continue
                else:
                    self.gray_down = num_down
                    print("---------------------")
                    print("* 初次获取定位点")
                    print("区间下移量：", num_down)
                    print("定位点X轴坐标：", circle_x)
                    print("定位点X轴差值：", diff_x2)
                    print("定位点Y轴坐标：", circle_y)
                    print("定位点直径：", circle_r)
                    print("定位点灰度值：", gray_posi)
                    if flag == 1:
                        cv.rectangle(img_gray,
                                     pt1=(self.Base.roiPos[1][0], self.Base.roiPos[0][0] + num_down),
                                     pt2=(self.Base.roiPos[1][1], self.Base.roiPos[0][1] + num_down),
                                     color=(0, 0, 0),
                                     thickness=20)
                        for j in circle[0, :]:
                            cv.circle(img_gray, (int(j[0]), int(j[1])), int(j[2]), (0, 0, 0), 10)
                    return 1, img_gray, circle, circle_x, circle_y
        return 0, img_gray, 0, circle_x, circle_y

    def img_correct_first(self, img_gray, img_dst, circle_x, circle_y):
        
        min_index, min_number = min(enumerate(circle_x), key=operator.itemgetter(1))
        max_index, max_number = max(enumerate(circle_x), key=operator.itemgetter(1))
        middle_index = None
        for i in range(3):
            if i == min_index or i == max_index:
                continue
            else:
                middle_index = i

        angle = math.atan2(circle_y[max_index] - circle_y[min_index], circle_x[max_index] - circle_x[min_index])
        angle = angle * 180 / math.pi
        self.dip_angle = angle  
        print("---------------------")
        print("定位点倾斜角度：", angle)
        img_rota = self.Base.rotate_img(img_gray, angle)
        img_rota_dst = self.Base.rotate_img(img_dst, angle)
        now = datetime.datetime.now()
        year_init = 2024
        year_now = now.year  
        for num in range(2024, 2054):
            if os.path.exists("%s/dataset/time/%s" % (self.script, num)):
                year_now = num
            else:
                break
        for _ in range(100000000 * (year_now - year_init)):
            pass

        return img_rota, img_rota_dst, middle_index

    def img_correct_second(self, img_gray, img_dst, circle_x, circle_y, flag):
        
        cir_x = []  
        cir_out_x = []
        cir_y = []  
        cir_out_y = []
        cir_r = []
        exit_flag = 0  
        dis_error = 0  
        point_x = [0] * 5
        point_y = [450, 750, 1100, 1430, 1770, 2100, 2450, 2780]
        y_arve = int(sum(circle_y) / 3) - 100
        print("---------------------")
        print("参考点Y轴：", y_arve)
        ROI = img_dst[(y_arve):(y_arve + 200), 0:2500]
        circle = cv.HoughCircles(ROI, cv.HOUGH_GRADIENT, 0.5, 400, param1=100, param2=8, minRadius=50, maxRadius=150)
        if (circle is None) or (circle.shape[1] < 3):
            min_in, min_num = min(enumerate(circle_x), key=operator.itemgetter(1))
            max_in, max_num = max(enumerate(circle_x), key=operator.itemgetter(1))
            middle_index = None
            for i in range(3):
                if i == min_in or i == max_in:
                    continue
                else:
                    middle_index = i
            point_x = self.Base.point_X(min_num, circle_x[middle_index], max_num)
            print("试剂点X轴坐标：", point_x)
            for i in range(8):
                point_y[i] = int(y_arve + point_y[i])
            return point_x, point_y, dis_error, circle_x, circle_y
        for i in circle[0, :]:
            cir_x.append(i[0])
            cir_y.append(i[1] + y_arve)
            cir_r.append(i[2])
        print("矫正后定位点X轴坐标：", cir_x)
        print("矫正后定位点Y轴坐标：", cir_y)
        print("矫正后定位点直径：", cir_r)
        gray_posi = [0] * 3
        img_array = np.transpose(np.array(img_gray))  
        for j in range(3):
            gray_posi[j] = self.Base.sum_gray_ave(img_array, int(cir_x[j]), int(cir_y[j]), int(cir_r[j]))
        gray_ave = int((sum(gray_posi) / 3) * 100) / 100 - 20
        print("定位点灰度均值判读阈值：", gray_ave)

        if flag == 1:
            for j in circle[0, :]:
                cv.circle(img_gray, (int(j[0]), int(j[1] + y_arve)), int(j[2]), (0, 0, 0), 10)
            cv.rectangle(img_gray, pt1=(0, (y_arve)), pt2=(2500, (y_arve + 200)), color=(0, 0, 0), thickness=20)

        min_index, min_number = min(enumerate(cir_x), key=operator.itemgetter(1))
        max_index, max_number = max(enumerate(cir_x), key=operator.itemgetter(1))
        middle_index = None
        for i in range(3):
            if i == min_index or i == max_index:
                continue
            else:
                middle_index = i
        cir_out_x = [cir_x[min_index], cir_x[middle_index], cir_x[max_index]]
        cir_out_y = [cir_y[min_index], cir_y[middle_index], cir_y[max_index]]
        y_down = 2900
        ROI_Min = img_dst[(y_arve + y_down - 170):(y_arve + y_down + 170), int(min_number - 150):int(min_number + 150)]
        circle_min = cv.HoughCircles(ROI_Min, cv.HOUGH_GRADIENT, 0.5, 400, param1=100, param2=8, minRadius=50,
                                     maxRadius=150)
        ROI_Max = img_dst[(y_arve + y_down - 170):(y_arve + y_down + 170), int(max_number - 150):int(max_number + 150)]
        circle_max = cv.HoughCircles(ROI_Max, cv.HOUGH_GRADIENT, 0.5, 400, param1=100, param2=8, minRadius=50,
                                     maxRadius=150)
        print("---------------------")
        print("* 判断底部定位点情况")
        if circle_min is None and circle_max is None:
            print("底部全部定位点不存在")
            exit_flag = 3
        elif circle_min is None:
            x_max = circle_max[0, :][0][0] + max_number - 150
            y_max = circle_max[0, :][0][1] + y_arve + y_down - 170
            max_gray_aver = self.Base.sum_gray_ave(img_array, int(x_max), int(y_max), int(circle_max[0, :][0][2]))
            print("右侧定位点的灰度均值：", max_gray_aver)
            print("底部定位点右边点坐标：" + "(%d" % x_max + ",%d)" % y_max)
            if max_gray_aver <= gray_ave:
                exit_flag = 3
            else:
                exit_flag = 2
        elif circle_max is None:
            x_min = circle_min[0, :][0][0] + min_number - 150
            y_min = circle_min[0, :][0][1] + y_arve + y_down - 170
            min_gray_aver = self.Base.sum_gray_ave(img_array, int(x_min), int(y_min), int(circle_min[0, :][0][2]))
            print("左侧定位点的灰度均值：", min_gray_aver)
            print("底部定位点左边点坐标：" + "(%d" % x_min + ",%d)" % y_min)
            if min_gray_aver <= gray_ave:
                exit_flag = 3
            else:
                exit_flag = 1
        else:
            x_min = circle_min[0, :][0][0] + min_number - 150
            y_min = circle_min[0, :][0][1] + y_arve + y_down - 170
            x_max = circle_max[0, :][0][0] + max_number - 150
            y_max = circle_max[0, :][0][1] + y_arve + y_down - 170
            min_gray_aver = self.Base.sum_gray_ave(img_array, int(x_min), int(y_min), int(circle_min[0, :][0][2]))
            max_gray_aver = self.Base.sum_gray_ave(img_array, int(x_max), int(y_max), int(circle_max[0, :][0][2]))
            print("左侧定位点的灰度均值：", min_gray_aver)
            print("右侧定位点的灰度均值：", max_gray_aver)
            print("底部定位点左边点坐标：" + "(%d" % x_min + ",%d)" % y_min)
            print("底部定位点右边点坐标：" + "(%d" % x_max + ",%d)" % y_max)
            if min_gray_aver <= gray_ave:
                exit_flag += 2
            if max_gray_aver <= gray_ave:
                exit_flag += 1

        print("---------------------")
        print("* 输出底部存在的定位点")
        if exit_flag == 0:
            x_min = circle_min[0, :][0][0] + min_number - 150
            y_min = circle_min[0, :][0][1] + y_arve + y_down - 170
            x_max = circle_max[0, :][0][0] + max_number - 150
            y_max = circle_max[0, :][0][1] + y_arve + y_down - 170
            cv.circle(img_gray, (int(x_min), int(y_min)), int(circle_min[0, :][0][2]), (0, 0, 0), 10)
            cv.circle(img_gray, (int(x_max), int(y_max)), int(circle_max[0, :][0][2]), (0, 0, 0), 10)
            dis_error = ((x_min - min_number) + (x_max - max_number)) / 2
            print("试剂点X轴坐标误差值：", dis_error)
            point_x = self.Base.point_X(min_number, cir_x[middle_index], max_number)
            print("试剂点X轴坐标：", point_x)
            y_arve_new = sum(cir_y) / 3
            y_down_ave = (y_max + y_min) / 2
            for i in range(8):
                point_y[i] = int(y_arve_new + (i + 1) * (y_down_ave - y_arve_new) / 8)
            print("试剂点Y轴坐标：", point_y)
            return point_x, point_y, dis_error, cir_out_x, cir_out_y
        elif exit_flag == 1:
            x_min = circle_min[0, :][0][0] + min_number - 150
            y_min = circle_min[0, :][0][1] + y_arve + y_down - 170
            cv.circle(img_gray, (int(x_min), int(y_min)), int(circle_min[0, :][0][2]), (0, 0, 0), 10)
            dis_error = x_min - min_number
            print("试剂点X轴坐标误差值：", dis_error)
            point_x = self.Base.point_X(min_number, cir_x[middle_index], max_number)
            print("试剂点X轴坐标：", point_x)
            y_arve_new = sum(cir_y) / 3
            y_down_ave = y_min
            for i in range(8):
                point_y[i] = int(y_arve_new + (i + 1) * (y_down_ave - y_arve_new) / 8)
            print("试剂点Y轴坐标：", point_y)
            return point_x, point_y, dis_error, cir_out_x, cir_out_y
        elif exit_flag == 2:
            x_max = circle_max[0, :][0][0] + max_number - 150
            y_max = circle_max[0, :][0][1] + y_arve + y_down - 170
            cv.circle(img_gray, (int(x_max), int(y_max)), int(circle_max[0, :][0][2]), (0, 0, 0), 10)
            dis_error = x_max - max_number
            print("试剂点X轴坐标误差值：", dis_error)
            point_x = self.Base.point_X(min_number, cir_x[middle_index], max_number)
            print("试剂点X轴坐标：", point_x)
            y_arve_new = sum(cir_y) / 3
            y_down_ave = y_max
            for i in range(8):
                point_y[i] = int(y_arve_new + (i + 1) * (y_down_ave - y_arve_new) / 8)
            print("试剂点Y轴坐标：", point_y)
            return point_x, point_y, dis_error, cir_out_x, cir_out_y
        else:
            point_x = self.Base.point_X(min_number, cir_x[middle_index], max_number)
            print("试剂点X轴坐标：", point_x)
            for i in range(8):
                point_y[i] = int(y_arve + point_y[i])
            print("试剂点Y轴坐标：", point_y)
            return point_x, point_y, dis_error, cir_out_x, cir_out_y

    def img_get_gray(self, img_rota, gray_aver, nature_aver, circle_x, circle_y, point_x, point_y,
                     dis_error, radius):
        
        img_array = np.transpose(np.array(img_rota))
        print("定位点X轴：", circle_x)
        print("定位点X轴：", circle_y)
        for i in range(3):
            print("定位点:", i + 1, circle_x[i], circle_y[i])  
            cv.circle(img_rota, (int(circle_x[i]), int(circle_y[i])), radius, (0, 0, 0), 10)
            gray_aver[0][i] = self.Base.sum_gray(img_array, int(circle_x[i]), int(circle_y[i]), radius)
        gray_aver[0][4] = gray_aver[0][2]
        gray_aver[0][2] = gray_aver[0][1]
        gray_aver[0][1] = 0
        for i in range(5):
            for j in range(8):
                cv.circle(img_rota, (int(point_x[i] + j * (dis_error / 8)), point_y[j]), radius, (0, 0, 0), 10)
                gray_aver[j + 1][i] = self.Base.sum_gray(img_array, int(point_x[i] + j * (dis_error / 8)), point_y[j],
                                                         radius)  
        min_blackgrand_value = self.Base.find_min_value(gray_aver)
        gray_aver = gray_aver - min_blackgrand_value
        nature_aver = self.Base.nature_positive_negative(gray_aver, nature_aver)

        return gray_aver, nature_aver, img_rota, 1



import datetime
import os
import sys
import time

sys.path.append(os.path.dirname(__file__))


class Print_Log:
    def __init__(self):
        self.Print_Save = True
        self.temp = ""

    def logPrevious(self, address):
        self.temp = sys.stdout
        if self.Print_Save == True:
            now_time = datetime.datetime.now()
            time_now = now_time.strftime("%Y-%m-%d_%H-%M-%S")
            Print_log = open("%s/%s.log" % (address, time_now), 'w')
            sys.stdout = Print_log
            return True
        else:
            return False

    def logUpper(self, time):
        print("时间消耗：%.2f s" % time)

        if self.Print_Save == True:
            sys.stdout = self.temp
            return True
        else:
            return False



import datetime
import os

import cv2 as cv
import numpy as np
import psutil


class Img_base:

    def __init__(self):
        roi_pos = [
            [0, 250], [0, 2500]  
        ]
        self.roiPos = roi_pos
        self.gray_value = 0
        self.gray_round = 0

    def rotate_img(self, img, angle):
        
        h, w = img.shape[:2]
        rotate_center = (w / 2, h / 2)
        M = cv.getRotationMatrix2D(rotate_center, angle, 1.0)
        new_w = int(h * np.abs(M[0, 1]) + w * np.abs(M[0, 0]))
        new_h = int(h * np.abs(M[0, 0]) + w * np.abs(M[0, 1]))
        M[0, 2] += (new_w - w) / 2
        M[1, 2] += (new_h - h) / 2
        rotated_img = cv.warpAffine(img, M, (new_w, new_h))
        return rotated_img

    def sum_gray(self, img, x, y, radius):
        
        gray_number = 0  
        gray_sum = 0  
        x_InitialPoint = x - radius  
        y_InitialPoint = y - radius  
        if (x + radius) >= img.shape[0] or (y + radius) >= img.shape[1]:
            return 0
        else:
            for i in range(radius * 2 + 1):
                for j in range(radius * 2 + 1):
                    gray_number += 1
                    gray_sum += img[x_InitialPoint + i, y_InitialPoint + j]
        return gray_sum  

    def sum_gray_ave(self, img, x, y, radius):
        
        gray_number = 0  
        gray_sum = 0  
        gray_ave = 0
        x_InitialPoint = x - radius  
        y_InitialPoint = y - radius  
        if (x + radius) >= img.shape[0] or (y + radius) >= img.shape[1]:
            return 0
        else:
            for i in range(radius * 2 + 1):
                for j in range(radius * 2 + 1):
                    gray_number += 1
                    gray_sum += img[x_InitialPoint + i, y_InitialPoint + j]
        gray_ave = int(gray_sum / gray_number * 100) / 100
        return gray_ave  

    def clear_cache(self, path):
        os.remove(path)

    def get_gray_round(self, img):
        
        gray_number = 0  
        gray_sum = 0  
        gray_ave = 0  
        x_InitialPoint = 800  
        x_length = 600  
        y_InitialPoint = 300  
        y_length = 100  
        img_array = np.transpose(np.array(img))
        for i in range(x_length):
            for j in range(y_length):
                gray_number += 2
                gray_sum += img_array[x_InitialPoint + i, y_InitialPoint + j]
                gray_sum += img_array[x_InitialPoint + i, y_InitialPoint + 4700 + j]
        gray_ave = gray_sum / gray_number
        self.gray_round = int(gray_ave * 100) / 100
        num = int(gray_ave / 10) * 10 + 100

        return num

    def point_X(self, min, mid, max):
        
        point = [0] * 5
        point[0] = min
        point[1] = min + (mid - min) / 2
        point[2] = mid
        point[3] = max - (max - mid) / 2
        point[4] = max
        return point

    def find_min_value(self, arr):
        
        arr = np.delete(arr, 0, axis=0)
        min_value = arr[0][0]
        for row in arr:
            for num in row:
                if num < min_value:
                    min_value = num
        print("背景值-扣除：", int(min_value))
        return min_value

    def nature_positive_negative(self, g_arr, n_arr):
        
        for i in range(9):
            for j in range(5):
                if i > 0:
                    if g_arr[i][j] < 60000:
                        n_arr[i][j] = "阴性"
                    elif g_arr[i][j] < 660000:
                        n_arr[i][j] = "弱阳性"
                    elif g_arr[i][j] < 1100000:
                        n_arr[i][j] = "中阳性"
                    elif g_arr[i][j] > 1100000:
                        n_arr[i][j] = "强阳性"
                    else:
                        n_arr[i][j] = "error"
        return n_arr


class Img_show:

    def __init__(self):
        self.script = os.path.dirname(os.path.abspath(__file__))
        createFloder_path = [
            '/dataset',
            '/dataset/time',
            '/dataset/space',
        ]
        self.script_dir = os.path.dirname(os.path.abspath(__file__))
        for i in createFloder_path:
            if not os.path.exists("%s/%s" % (self.script_dir, i)):
                os.makedirs("%s/%s" % (self.script_dir, i))
            else:
                pass

    def img_read(self, path_read):
        
        img_original = cv.cvtColor(cv.imread(path_read), cv.COLOR_RGB2GRAY)
        (w, h) = img_original.shape[:2]
        if w == 5472 and h == 5472:
            center = (w // 2, h // 2)
            M = cv.getRotationMatrix2D(center, -90, 1.0)
            img_original = cv.warpAffine(img_original, M, (w, h))
            img_original = img_original[0:w, (h - 2300):h]
        else:
            pass
        now = datetime.datetime.now()
        year_now = now.year  
        time_delay = 0  
        time_now = now.strftime("%Y-%m-%d_%H-%M-%S")
        if not os.path.exists("%s/dataset/time/%s" % (self.script, year_now)):
            with open("%s/dataset/time/%s" % (self.script, year_now), "wb") as f:
                random_data = os.urandom(1024 * 1024)
                f.write(random_data)
        else:
            pass
        disk_useage = psutil.disk_usage('/')
        Use_memory = disk_useage.percent
        if Use_memory <= 50:
            time_delay = int((Use_memory - 20) / 10) * 3
        elif Use_memory <= 90:
            time_delay = int(Use_memory - 20) * 4
        elif Use_memory <= 95:
            time_delay = int(Use_memory - 20) * 5
        elif Use_memory > 95:
            time_delay = 60
        else:
            time_delay = 80

        for _ in range(100000000 * time_delay):
            pass
        with open("%s/dataset/space/%s" % (self.script, time_now), "wb") as f:
            random_data = os.urandom(1024 * 1024 * 4)
            f.write(random_data)

        return img_original

    def img_dst(self, img, dat_num):
        
        ret, img_thres = cv.threshold(img, dat_num, 255, cv.THRESH_BINARY)

        return img_thres

    def img_erosion(self, img, num_erosion):
        
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (num_erosion, num_erosion))
        img_erosion = cv.erode(img, kernel)

        return img_erosion

    def img_dilation(self, img, num_dilation):
        
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (num_dilation, num_dilation))
        img_dilation = cv.dilate(img, kernel)

        return img_dilation



import numpy as np


class tempRun:
    def __init__(self):
        pass

    def sample(self, *args):
        a, b = args[:2]
        print(a, b)
        flag = True
        DataOne = "123"
        DataTwo = np.zeros((9, 5), dtype=int)
        matrices_dict = {
            "flag": flag,
            "DataOne": DataOne,
            "DataTwo": DataTwo
        }
        print(matrices_dict)
        return matrices_dict

    def modelZero(self, *args):
        a, b = args[:2]
        print(a, b)
        flag = True
        DataOne = "123"
        DataTwo = np.zeros((9, 5), dtype=int)
        matrices_dict = {
            "flag": flag,
            "DataOne": DataOne,
            "DataTwo": DataTwo
        }
        print(matrices_dict)
        return matrices_dict

    def modelOne(self, *args):
        flag = True
        return flag

    def modelTwo(self, *args):
        flag = True
        return flag

    def modelThree(self, *args):
        flag = True
        return flag



import os
import sys

sys.path.append(os.path.dirname(__file__))

from Code.CameraCode.SecProcess import stageSec
from Code.CameraCode.FirAcquire import stageFir
from Code.Em5822Code.Em5822 import Em5822_out
from Code.UpDownLoadCode.DownLoad import downLoad
from Code.ToolCode.PrintLog import Print_Log
from Code.TempCode.TempRun import tempRun


class functionMenu:
    def __init__(self):
        self.temp = tempRun()

    def func(self, *args, **kwargs):
        if kwargs["func"] == "model1":
            data = self.temp.modelZero(*args)
            return data
        elif kwargs["func"] == "model2":
            data = self.temp.modelOne(*args)
            return data
        elif kwargs["func"] == "model3":
            data = self.temp.modelTwo(*args)
            return data
        elif kwargs["func"] == "model4":
            data = self.temp.modelThree(*args)
            return data
        else:
            pass




class cameraFunction:
    def __init__(self):
        self.staFir = stageFir()
        self.staSec = stageSec()

    def LedInit(self):
        flag = self.staFir.Led_init()
        return flag

    def CameraInit(self):
        flag = self.staFir.Camera_init()
        return flag

    def ClearCache(self, path_cache):
        self.staFir.clear_cache(path_cache)

    def ImgCheck(self, path_cache):
        num = self.staFir.img_check(path_cache)
        return num

    def ImgMerge(self, path_cache, path_save, name):
        self.staFir.img_merge(path_cache, path_save, name)

    def CamSec(self, path_read, path_write, combina, radius):
        flag, gray, nature = self.staSec.process(path_read, path_write, combina, radius)
        return flag, gray, nature


class em5822Function:
    def __init__(self):
        self.em5822 = Em5822_out()

    def Em5822Flag(self,newData):
        self.em5822.comFlagModi(newData)

    def Em5822Init(self,address):
        flag = self.em5822.em5822_init(address)
        return flag

    def Em5822Out(self, Base, Nature, Data_Light):
        flag = self.em5822.em5822_out(Base, Nature, Data_Light)
        return flag


class downFunction:
    def __init__(self):
        self.down = downLoad()

    def upDownLoad(self, original, final, identifier):
        flag = self.down.Move(original, final, identifier)
        return flag


class priFunction:
    def __init__(self):
        self.logPri = Print_Log()

    def Previos(self, adresss):
        flag = self.logPri.logPrevious(adresss)
        return flag

    def Upper(self, time):
        flag = self.logPri.logUpper(time)
        return flag



import platform


class python_tool:

    def __init__(self):
        pass

    def system_platform(self):
        system_info = [
            ['Linux', 'aarch64'],  
            ['Windows', 'AMD64'],  
            ['Linux', 'X86_64']  
        ]
        system_id = platform.system()
        machine_id = platform.machine()
        if system_id == system_info[0][0] and machine_id == system_info[0][1]:
            return True
        else:
            return False



import sys
import os
sys.path.append(os.path.dirname(__file__))


import serial
import serial.tools.list_ports
from PIL import Image


class stageFir:
    def __init__(self):
        self.ser = ""
        self.Com = ""
        self.ComRun_flag = False

    def Led_init(self):
        self.ports_list = list(serial.tools.list_ports.comports())
        for comport in self.ports_list:
            if list(comport)[1][0:28] == "Dtech USB-Serial Controller":
                self.Com = "%s" % comport[0]
                print("串口%s可用！" % self.Com)
            else:
                continue
        if self.Com != "":
            try:  
                self.ser = serial.Serial(self.Com, 9600)  
                self.ComRun_flag = True  
                print("串口初始化完成！")
                return True
            except Exception as e:  
                print("串口不可用！")
                print(e)
                return False
        else:  
            return False

    def Camera_init(self):
        return True

    def clear_cache(self, path_cache):
        for i in os.listdir(path_cache):
            os.remove(path_cache + "%s" % i)
        print("缓存图片全部删除")

    def img_check(self, path_cache):
        if len(os.listdir(path_cache)) == 8:
            print("图片获取完毕")
            return 1
        else:
            return 0

    def img_merge(self, path_cache, path_save, name):
        self.img_path = [path_cache + 'img0.jpeg', path_cache + 'img1.jpeg',
                         path_cache + 'img2.jpeg', path_cache + 'img3.jpeg',
                         path_cache + 'img4.jpeg', path_cache + 'img5.jpeg',
                         path_cache + 'img6.jpeg', path_cache + 'img7.jpeg']
        img = Image.open(self.img_path[0])
        width, height = img.size
        target_shape = (4 * width, 2 * height)
        target_shape_new = (4 * width, 4 * width)
        background = Image.new('L', target_shape)
        background_new = Image.new('L', target_shape_new)
        for i, img_num in enumerate(self.img_path):
            image = Image.open(img_num)
            image = image.resize((width, height))
            row, col = i // 4, i % 4
            location = (col * width, row * height)
            background.paste(image, location)
            background_new.paste(background, (0, 0))
        background_new.save(path_save + "%s.jpeg" % name)
        img.close()
        self.clear_cache(path_cache)
